// gm api implement file
// detail: https://www.tampermonkey.net/documentation.php

(function() {
    if (window.XGM == null) {
       function _GM_hashCode(str) {
           var hash = 0, i, chr;
           if (str == null || str.length === 0) {
               return hash;
           }

           for (i = 0; i < str.length; i++) {
               chr = str.charCodeAt(i);
               hash  = ((hash << 5) - hash) + chr;
               hash |= 0; // Convert to 32bit integer
           }

           return hash;
       };

       function _GM_localStorage_removeItem(key) {
           if (XWebViewApi != null && XWebViewApi.localStorage != null) {
               XWebViewApi.localStorage.removeItem(key);
           } else {
               localStorage.removeItem(key);
           }
       }

       function _GM_localStorage_getItem(key) {
           if (XWebViewApi != null && XWebViewApi.localStorage != null) {
               return XWebViewApi.localStorage.getItem(key);
           } else {
               return localStorage.getItem(key);
           }
       }

       function _GM_localStorage_setItem(key, value) {
           if (XWebViewApi != null && XWebViewApi.localStorage != null) {
               XWebViewApi.localStorage.setItem(key, value);
           } else {
               localStorage.setItem(key, value);
           }
       }

       function _GM_localStorage_key(idx) {
           if (XWebViewApi != null && XWebViewApi.localStorage != null) {
               XWebViewApi.localStorage.key(idx);
           } else {
               localStorage.key(idx);
           }
       }

       function _GM_localStorage_length() {
           if (XWebViewApi != null && XWebViewApi.localStorage != null) {
               return XWebViewApi.localStorage.size();
           } else {
               return localStorage.length;
           }
       }

       function _GM_fireValueChange(aKey, old_value, new_value, remote) {
           if (window._GM_valueCbs != null) {
               for (var i = 0; i < window._GM_valueCbs.length; ++i) {
                   var item = window._GM_valueCbs[i];
                   if (item.aKey == aKey) {
                       item.cb(aKey, old_value, new_value, remote);
                   }
               }
           }
       }

       function _GM_storagePrefix(gmInfo) {
           return "gm_" + gmInfo.uuid + "_"
       }

       function _GM_setupRequestEvent(aOpts, aReq, aEventName) {
           if (!aOpts['on' + aEventName]) {
               return;
           }

           aReq.addEventListener(aEventName, function(aEvent) {
               var responseState = {
                    responseText: aReq.responseText,
                    responseXML: aReq.responseXML,
                    response: aOpts.responseType === 'json' && aReq.response != null ? JSON.parse(aReq.response) : aReq.response,
                    readyState: aReq.readyState,
                    responseHeaders: null,
                    status: null,
                    statusText: null,
                    finalUrl: null
               };

               switch (aEventName) {
                   case "progress":
                       responseState.lengthComputable = aEvent.lengthComputable;
                       responseState.loaded = aEvent.loaded;
                       responseState.total = aEvent.total;
                       break;
                   case "error":
                       break;
                   default:
                       if (4 != aReq.readyState) break;
                       responseState.responseHeaders = aReq.getAllResponseHeaders();
                       responseState.status = aReq.status;
                       responseState.statusText = aReq.statusText;
                       break;
               }

               aOpts['on' + aEventName](responseState);
           });
       }

       window.XGM = {
           addStyle: function(gmInfo, css) {
               var head = document.getElementsByTagName('head')[0];
               if (head) {
                   var style = document.createElement('style');
                   style.setAttribute('type', 'text/css');
                   style.textContent = css;
                   head.appendChild(style);
                   return style;
               }

               return null;
           },

           addElement: function(gmInfo, ...args) { // (tag_name, attributes) or (parent_node, tag_name, attributes)

           },

           deleteValue: function(gmInfo, name) {
               _GM_localStorage_removeItem(_GM_storagePrefix(gmInfo) + name);
           },

           listValues: function(gmInfo) {
               var prefix = _GM_storagePrefix(gmInfo);
               var prefixLen = prefix.length;
               var values = [];

               var size = _GM_localStorage_length();
               for (var i = 0; i < size; i++) {
                   var k = _GM_localStorage_key(i);

                   if (k.substr(0, prefixLen) === prefix) {
                       values.push(k.substr(prefixLen));
                   }
               }

               return values;
           },

           addValueChangeListener: function(gmInfo, name, callback) { // callback: function(name, old_value, new_value, remote) {}
               var item = {
                   id: 'gm_' + gmInfo.uuid + new Date().getTime(),
                   key: name,
                   cb: callback
               }

               if (window._GM_valueCbs == null) {
                   window._GM_valueCbs = [];
               }

               window._GM_valueCbs.push(item);
               return item.id;
           },

           removeValueChangeListener: function(gmInfo, id) {
               if (window._GM_valueCbs != null) {
                   for (var i = 0; i < window._GM_valueCbs.length; ++i) {
                       var item = window._GM_valueCbs[i];
                       if (item.id == id) {
                           window._GM_valueCbs.splice(i, 1);
                           break;
                       }
                   }
               }
           },

           setValue: function(gmInfo, name, value) {
               var val = _GM_localStorage_getItem(_GM_storagePrefix(gmInfo) + name);
               if (null === val) {
                   val = JSON.stringify('');
               }

               _GM_localStorage_setItem(_GM_storagePrefix(gmInfo) + name, JSON.stringify(value));
               _GM_fireValueChange(name, JSON.parse(val), value, '');
           },

           getValue: function(gmInfo, name, defaultValue) {
               var val = _GM_localStorage_getItem(_GM_storagePrefix(gmInfo) + name);

               if (null === val && 'undefined' != typeof defaultValue) {
                   return defaultValue;
               }

               return JSON.parse(val);
           },

           log: function(gmInfo, message) {
               console.log(message);
           },

           getResourceText: function(gmInfo, name) {
               return '';
           },

           getResourceURL: function(gmInfo, name) {
               return 'gm:' + gmInfo.uuid + '/' + name;
           },

           registerMenuCommand: function(gmInfo, name, fn, accessKey) {
               if (window.__GM_GC == null) {
                   window.__GM_GC = [];
               }

               var menuCmdId = 'menu:gm_' + _GM_hashCode(name) + '@' + gmInfo.uuid;
               var cbn = 'window.__GM_GC["' + menuCmdId + '"]';

               window.__GM_GC[menuCmdId] = function(d) {
                   fn();
               }

               if (XWebViewApi != null && XWebViewApi.registerMenuCommand != null) {
                   XWebViewApi.registerMenuCommand('menu.bar.top.gm', menuCmdId, name, cbn);
               }

               return menuCmdId;
           },

           unregisterMenuCommand: function(gmInfo, menuCmdId) {
               if (window.__GM_GC != null) {
                   window.__GM_GC[menuCmdId] = null;

                   if (XWebViewApi != null && XWebViewApi.registerMenuCommand != null) {
                       XWebViewApi.unregisterMenuCommand('menu.bar.top.gm', menuCmdId);
                   }
               }
           },

           openInTab: function(gmInfo, url, extra) { // (url, options) or (url, loadInBackground)
               window.location.href = url;
               //window.open(url);
           },

           xmlhttpRequest: function(gmInfo, aOpts) {
               var req = new XMLHttpRequest();
               _GM_setupRequestEvent(aOpts, req, 'abort');
               _GM_setupRequestEvent(aOpts, req, 'error');
               _GM_setupRequestEvent(aOpts, req, 'load');
               _GM_setupRequestEvent(aOpts, req, 'progress');
               _GM_setupRequestEvent(aOpts, req, 'readystatechange');

               req.open(aOpts.method, aOpts.url, !aOpts.synchronous, aOpts.user || '', aOpts.password || '');

               if (aOpts.overrideMimeType) {
                   req.overrideMimeType(aOpts.overrideMimeType);
               }

               if (aOpts.headers) {
                   for (let prop in aOpts.headers) {
                       if (Object.prototype.hasOwnProperty.call(aOpts.headers, prop)) {
                            try {
                                req.setRequestHeader(prop, aOpts.headers[prop]);
                            } catch (e) {
                                console.log("req.setRequestHeader error:" + e + ", prop:" + prop);
                            }
                       }
                   }
               }

               var body = aOpts.data ? aOpts.data : null;

               if (aOpts.binary) {
                   return req.sendAsBinary(body);
               } else {
                   return req.send(body);
               }
           },

           download: function(gmInfo, arg1, arg2) { //(details) or (url, name)
                if (typeof arg1 === 'string') {
                    window.location.href = arg1;
                }
           },

           getTab: function(gmInfo, callback) {
               callback({tabId: "default"});
           },

           saveTab: function(gmInfo, tab) {

           },

           getTabs: function(gmInfo, callback) {
               // 获取所有tab对象作为散列与其他脚本实例通信。
               callback([{tabId: "default"}]);
           },

           notification: function(gmInfo, details, ondone) { //(details, ondone) or (text, title, image, onclick)
               //    details 的属性：
               //    text - 通知的问题 如果高亮就 就不需要
               //    title - 通知的标题
               //    image - 图片
               //    highlight - 一个boolean标志，是否高亮tab
               //    silent -  一个boolean 是否播放音乐
               //    timeout - 通知显示的时间  0表示 一直显示
               //    ondone - 通知被关闭时 无论是被点击还是超时 执行的函数
               //    onclick - 点击通知触发的函数
               //    所有参数的作用与其对应的详细信息属性挂件完全相同。
               if (XWebViewApi != null && XWebViewApi.showNotification != null && details != null) {
                    var id = 'notify:gm_' + _GM_hashCode(details.title) + '@' + gmInfo.uuid;
                    var callback = '';

                    if (details.onclick != null) {
                         callback = 'window.__GM_GC["' + id + '"]';
                         window.__GM_GC[id] = function(d) {
                            details.onclick();
                         }
                    }

                    XWebViewApi.showNotification('app.browser.gm.notify', id, details.title, details.text, details.image, details.timeout, callback);
               }
           },

           setClipboard: function(gmInfo, data, info) {
               //复制数据到粘贴板，参数info可以是对象如{type: 'text', mimetype: 'text/plain'}，或者是一个字符串 text， html
               var aux = document.createElement('input')
               aux.setAttribute('value', data)
               document.body.appendChild(aux)
               aux.select()
               document.execCommand('copy')
               document.body.removeChild(aux)
           },
       };
    }
})();
